using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.GameContent;

namespace RoomIndicatorMod {
	public class RoomIndicatorSystem: ModSystem {
		protected const int listenInterval = 1000;

		protected RoomIndicatorRenderer iconRenderer;
		protected bool inside;
		protected bool cellar;

		private ICoreClientAPI capi;
		private long listenerId;

		public bool isInside() {
			return this.inside;
		}

		public bool isCellar() {
			return this.cellar;
		}

		public override bool ShouldLoad(EnumAppSide side) {
			return side == EnumAppSide.Client;
		}

		public override void StartClientSide(ICoreClientAPI capi) {
			this.capi = capi;
			this.listenerId = this.capi.Event.RegisterGameTickListener(this.OnGameTick, listenInterval);
			this.iconRenderer = new RoomIndicatorRenderer(capi, this);

			this.capi.Event.RegisterRenderer(this.iconRenderer, EnumRenderStage.Ortho);
		}

		public override void Dispose() {
			base.Dispose();

			this.capi.Event.UnregisterRenderer(this.iconRenderer, EnumRenderStage.Ortho);
			this.capi.Event.UnregisterGameTickListener(this.listenerId);
		}

		private void OnGameTick(float dt) {
			// Update icon.
			EntityPlayer entity = this.capi.World.Player.Entity;
			if(entity == null) {
				this.inside = false;
				this.cellar = false;
				return;
			}

			Room room = entity.World.Api.ModLoader.GetModSystem<RoomRegistry>().GetRoomForPosition(entity.Pos.AsBlockPos);
			if(room == null) {
				this.inside = false;
				this.cellar = false;
				return;
			}

			if(room.ExitCount == 0) {
				// Inside.
				this.inside = true;
				this.cellar = room.IsSmallRoom;
			} else {
				// Outside.
				this.inside = false;
				this.cellar = false;
			}
		}
	}

	public class RoomIndicatorRenderer: IRenderer {
		protected const float iconX = 452;		// Offset from center.
		protected const float iconY = 12;		// Offset from bottom.
		protected const float iconWidth = 32;
		protected const float iconHeight = 32;

		protected ICoreClientAPI capi;
		protected RoomIndicatorSystem risys;
		protected LoadedTexture roomTexture;
		protected LoadedTexture cellarTexture;

		public double RenderOrder {
			get {
				return 0;
			}
		}
		public int RenderRange {
			get {
				return 0;
			}
		}

		public RoomIndicatorRenderer(ICoreClientAPI capi, RoomIndicatorSystem risys) {
			this.capi = capi;
			this.risys = risys;
			this.roomTexture = new LoadedTexture(capi);
			this.cellarTexture = new LoadedTexture(capi);
			
			this.capi.Render.GetOrLoadTexture(new AssetLocation("roomindicator", "textures/room.png"), ref this.roomTexture);
			this.capi.Render.GetOrLoadTexture(new AssetLocation("roomindicator", "textures/cellar.png"), ref this.cellarTexture);
		}

		public void OnRenderFrame(float deltaTime, EnumRenderStage stage) {
			if(!this.risys.isInside()) {
				return;
			}

			float width = (float)GuiElement.scaled(iconWidth);
			float height = (float)GuiElement.scaled(iconHeight);

			float x = (this.capi.Render.FrameWidth / 2) - (width / 2) + (float)GuiElement.scaled(iconX);
			float y = (this.capi.Render.FrameHeight - height) - (float)GuiElement.scaled(iconY);
			
			this.capi.Render.RenderTexture(this.risys.isCellar() ? this.cellarTexture.TextureId : this.roomTexture.TextureId, x, y, width, height);
		}

		public void Dispose() {
			this.roomTexture.Dispose();
			this.cellarTexture.Dispose();
		}
	}
}